<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPoolPlugin\EventListener;

use Dedi\SyliusCustomerPoolPlugin\Model\Customer\CustomerPoolAwareInterface;
use Dedi\SyliusCustomerPoolPlugin\Repository\UserWithCustomerPoolRepositoryInterface;
use Sylius\Component\Channel\Context\ChannelContextInterface;
use Symfony\Component\EventDispatcher\GenericEvent;

final class CustomerPoolSetterListener
{
    private ChannelContextInterface $channelContext;

    public function __construct(ChannelContextInterface $channelContext)
    {
        $this->channelContext = $channelContext;
    }

    public function setPool(GenericEvent $event): void
    {
        /** @var CustomerPoolAwareInterface $customer */
        $customer = $event->getSubject();

        if (!$customer instanceof CustomerPoolAwareInterface) {
            throw new \InvalidArgumentException(sprintf(
                'Expected an instance of %s. Got: %s',
                UserWithCustomerPoolRepositoryInterface::class,
                get_class($customer)
            ));
        }

        /** @var CustomerPoolAwareInterface $channel */
        $channel = $this->channelContext->getChannel();

        if (!$channel instanceof CustomerPoolAwareInterface) {
            throw new \InvalidArgumentException(sprintf(
                'Expected an instance of %s. Got: %s',
                CustomerPoolAwareInterface::class,
                get_class($channel)
            ));
        }

        $customer->setCustomerPool($channel->getCustomerPool());
    }
}
